from torch.nn import Module
from torch.nn import Conv2d, Linear
from torch.nn.functional import relu, max_pool2d

class Lenet5(Module):
    def __init__(self, class_num=10):
        super(Lenet5, self).__init__()
        # 定义运算对象 n * 1 * 28 * 28
        # conv1 是第一个卷积层，输入大小为 n * 1 * 28 * 28，其中 n 表示样本的数量，1 表示输入的通道数（灰度图像），
        # 28 * 28 表示输入的图像大小。输出大小为 n * 6 * 14 * 14，其中 6 表示输出的通道数，14 * 14 表示输出的图像大小（通过 2x2 的最大池化操作减半）。
        self.conv1 = Conv2d(in_channels=1, out_channels=6, kernel_size=5, stride=1, padding=2)
        # n * 6 * 28 * 28
        # n * 6 * 14 * 14
        # conv2 是第二个卷积层，输入大小为 n * 6 * 14 * 14，输出大小为 n * 16 * 5 * 5。
        # 通过 2x2 的最大池化操作再次减半了输出的图像大小
        self.conv2 = Conv2d(in_channels=6, out_channels=16, kernel_size=5, stride=1, padding=0)
        # n * 16 * 10 * 10
        # n * 16 * 5 * 5
        # conv3 是第三个卷积层，输入大小为 n * 16 * 5 * 5，输出大小为 n * 120 * 1 * 1。
        # 这一层的卷积核大小设置为 5x5，没有进行池化操作。
        self.conv3 = Conv2d(in_channels=16, out_channels=120, kernel_size=5, stride=1, padding=0)
        # n * 120 * 1 * 1
        # 图像转换为向量
        # fc1 是第一个全连接层，输入大小为 n * 120，输出大小为 n * 84。
        self.fc1 = Linear(120, 84)
        # fc2 是第二个全连接层，输入大小为 n * 84，输出大小为 n * class_num，其中 class_num 表示分类的类别数量。
        self.fc2 = Linear(84, class_num)

    def forward(self, x):
        '''
        前向传播（Forward Propagation）：
            前向传播是神经网络中的信息传递过程，从输入层经过各层的人工神经元计算，最终得到输出结果。
            在前向传播过程中，每个神经元根据输入和权重计算输出，并将输出传递给下一层
            forward 函数是神经网络模型类中的一个方法，用于定义模型的前向传播过程。
            在 PyTorch 中，所有继承自 torch.nn.Module 的类都必须实现 forward 方法。
            forward 方法接受输入数据 x，并通过模型的各个层进行前向计算，最终得到输出结果。
        '''
        y = x
        # 应用第一个卷积层，对输入 y 进行卷积操作。
        y = self.conv1(y)
        # 对经过卷积后的特征图 y 进行最大池化操作，缩小特征图的空间尺寸。
        y = max_pool2d(y, (2, 2))
        # 应用 ReLU 激活函数，对特征图 y 进行非线性变换。
        y = relu(y)
        # 重复步骤上面操作，应用第二个卷积层和第三个卷积层，并对每个卷积层后的特征图进行最大池化和 ReLU 激活函数操作。
        y = self.conv2(y)
        y = max_pool2d(y, (2, 2))
        y = relu(y)
        # 重复步骤上面操作，应用第二个卷积层和第三个卷积层，并对每个卷积层后的特征图进行最大池化和 ReLU 激活函数操作。
        y = self.conv3(y)
        y = relu(y)
        # 将最后一个卷积层的特征图 y 转换为一维张量，以便输入到全连接层。
        y = y.view(-1, 120)
        # 应用第一个全连接层，对输入 y 进行线性变换。
        y = self.fc1(y)
        # 应用 ReLU 激活函数，对全连接层的输出 y 进行非线性变换。
        y = relu(y)
        # 应用第二个全连接层，对输入 y 进行线性变换。
        y = self.fc2(y)
        return y
